
APP_ID_ANDROID = "ANDROID_APP"
APP_ID_IPHONE = "IPHONE_APP"
APP_ID_SIMPLE_BANKING = "SIMPLE_BANKING"

ACCESS_EDENTIFIER1 = "EDENTIFIER1"
ACCESS_EDENTIFIER2_CONNECTED = "EDENTIFIER2_CONNECTED"
ACCESS_EDENTIFIER2_UNCONNECTED = "EDENTIFIER2_UNCONNECTED"
ACCESS_SOFTTOKEN = "SOFTTOKEN"
ACCESS_BOUNDDEVICE_USERPIN = "BOUNDDEVICE_USERPIN"
ACCESS_BOUNDDEVICE_TOUCHIDPIN = "BOUNDDEVICE_TOUCHIDPIN"
ACCESS_SESSIONHANDOVER = "SESSIONHANDOVER"
ACCESS_OOBGENERIC = "OOBGENERIC"
